////////////////////////////////////////////////////////////////////////////////
//
// Main module for the EFFIC program
//
// This program reads a file containing two vectors, which represent "before
// cut" and "after cut" histograms. It then uses a Bayesian analysis to
// determine the posterior probability for the cut efficiency in each bin.
// A flat prior is used for the efficiency. The outputs calculated from the
// posterior probability are the most probable value of the efficiency, and
// the upper and lower edges of the shortest C% confidence interval; C is
// entered on the command line
//
// The calling syntax is:
//     EFFIC datafile conf
// where
//     datafile is the name of the data file from which we read
//     conf     is required probability content of the confidence interval
//
// Created 14 Feb 1996        Marc Paterno
////////////////////////////////////////////////////////////////////////////////

#include "effic/effic.hpp"

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <TChain.h>
#include <TROOT.h>
#include "TH1F.h"
#include "TF1.h"
#include "TMath.h"
#include "TFile.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TGraphAsymmErrors.h"
#include "TPaveStats.h"
#include "TPostScript.h"

using namespace std;

void printUsage()
{
  cerr << "Syntax is: effic datafile conflevel\n";
}

void DivHistograms(TH1 *h1, TH1 *h2, TH1 *h, double *x, double *y, double *eyl, double *eyh) {
  // Calculate the efficiency bin by bin as the ratio of two histograms.
  // Errors on the efficiency are calculated properly.
  int nbins = h->GetNbinsX();

  // Check histogram compatibility
  if (nbins != h1->GetNbinsX() || nbins != h2->GetNbinsX()) {
    std::cout << "Attempt to divide histograms with different number of bins" << std::endl;
    return;
  }

  // Issue a Warning if histogram limits are different
  if (h->GetXaxis()->GetXmin() != h1->GetXaxis()->GetXmin() ||
      h->GetXaxis()->GetXmax() != h1->GetXaxis()->GetXmax() ){
    std::cout << "Attempt to divide histograms with different axis limits" << std::endl;
  }

  if (h->GetXaxis()->GetXmin() != h2->GetXaxis()->GetXmin() ||
      h->GetXaxis()->GetXmax() != h2->GetXaxis()->GetXmax() ){
    std::cout << "Attempt to divide histograms with different axis limits" << std::endl;
  }

  // Loop on bins (including underflows/overflows)
  int bin;
  int b1,b2;
  double effi,erro;
  for (bin=0;bin<=nbins+1;bin++) {

    b1  = (int) h1->GetBinContent(bin);
    b2  = (int) h2->GetBinContent(bin);

    double mode, low, high;
    if (b2) {
      double conflevel=0.683; //1 sigma confidence level
      effic(b1,b2, conflevel, mode, low, high);
      effi=mode;
      erro = mode-low;
    } else {
      effi = 0.0001;
      erro = 0.0001;
    }
    if(erro<=0.001) erro=0.001;
    h->SetBinContent(bin,effi);
    h->SetBinError(bin,erro);

    if(h->GetBinCenter(bin)>2 && h->GetBinCenter(bin)<60) {
      x[bin]= h->GetBinCenter(bin);
      y[bin]=effi;
      eyl[bin]=mode-low;
      eyh[bin]=high-mode;
    }
  }
}

Double_t turnon(Double_t *x, Double_t *par)
{
  double halfpoint1 = par[0];
  double slope1     = par[1];
  //--------------
  double plateau2   = par[2];
  double halfpoint2 = par[3];
  double slope2     = par[4];
  double plateau3   = fabs(par[5]);
  double halfpoint3 = fabs(par[6]);
  double slope3     = par[7];
  double plateau4   = fabs(par[8]);
  //double halfpoint3 = 0.0;
  //double slope3     = 1.0;
  //double plateau4   = 0.0;
  //---------------

  double plateau1   = fabs(1.0-plateau2-plateau3-plateau4);
  double pt         = TMath::Max(x[0],0.000001);
  double arg1 = (pt - halfpoint1)/(TMath::Sqrt(pt)*slope1);
  double arg2 = (pt - halfpoint2)/(TMath::Sqrt(pt)*slope2);
  double arg3 = (pt - halfpoint3)/(TMath::Sqrt(pt)*slope3);
  double fitval = plateau4+0.5*plateau3*(1+TMath::Erf(arg3))+0.5*plateau2*(1+TMath::Erf(arg2))+0.5*plateau1*(1+TMath::Erf(arg1));
  return fitval;
}

int main(int argc, char* argv[])
{
  //TH1F *sht20_seta = (TH1F*)f->Get("A8_ec");
  //TH1F *allem_seta = (TH1F*)f->Get("allem_a8l8_leta");
  
  //18-APR-05
  gROOT->SetStyle("Plain"); 
  gStyle->SetOptTitle(1); 
  gROOT->ForceStyle(); 

  //gStyle->SetOptStat(1); 
  gStyle->SetOptFit(1); 
  //gStyle->SetStatX(0.65);
  //gStyle->SetStatY(0.3); 
  TCanvas *c1 = new TCanvas("c1","c1",700,400); 
  
  TFile *f = new TFile("presel.root");
  TPostScript *hist = new TPostScript("turnon.eps");
  TFile *target = TFile::Open( "turnon.root", "RECREATE" );

  char ntp[100];
  
  //for(int k=0;k<24;++k) {
  for(int k=0;k<3;++k) {
    hist->NewPage();
    sprintf(ntp,"A8_enumerator_%d",k);  TH1F *sht20_seta = (TH1F*)f->Get(ntp); 
    sprintf(ntp,"A8_denominator_%d",k); TH1F *allem_seta = (TH1F*)f->Get(ntp); 
   
    sht20_seta->Rebin(2); 
    allem_seta->Rebin(2); 
    TH1F *res = new TH1F("res","",80,0.,80.); 
    res->Rebin(2); 
    sht20_seta->Sumw2();
    allem_seta->Sumw2();
    
    //Division of histograms:
    //res->Divide(sht20_seta,allem_seta,1,1,"B"); 
    //int nbins=res->GetNbinsX()+1;
    int nbins=50;
    double x[nbins];
    double exl[nbins], exh[nbins]; 
    
    double y[nbins];
    double eyl[nbins], eyh[nbins]; 
    
    for(int i=0;i<nbins;++i) {x[i]=0;y[i]=0;exl[i]=1.0;exh[i]=1.0;eyl[i]=0;eyh[i]=0;}
    
    DivHistograms(sht20_seta,allem_seta,res,x,y,eyl,eyh);
    
    TGraphAsymmErrors *gr = new TGraphAsymmErrors(nbins,x,y,exl,exh,eyl,eyh);
    
    gr->SetMarkerStyle(20);
    gr->SetMarkerSize(0.5);
    gr->SetMinimum(0.);
    gr->SetMaximum(1.05);
    gr->GetXaxis()->SetRangeUser(0,60);
    gr->GetHistogram()->SetXTitle("Electron pT");
    gr->GetHistogram()->SetYTitle("Efficiency");
    gr->GetHistogram()->SetTitle(sht20_seta->GetTitle());
    
    gStyle->SetOptFit(1111111);
    Double_t par1[9];
    //Double_t init[9] = {32.,0.8,0.05,19.0,0.8,0.5,11.0,0.65,0.0};
    Double_t init[9] = {32.,0.8,0.42,19.0,0.8,0.5,8.78,0.39,0.0};
    
    TF1 *fitfunc = new TF1("fitfunc", turnon,2,60,9);
    fitfunc->SetParameter(0,init[0]); 
    //fitfunc->FixParameter(1,init[1]); 
    fitfunc->SetParameter(1,init[1]); 
    fitfunc->SetParameter(2,init[2]); 
    fitfunc->SetParameter(3,init[3]); 
    fitfunc->SetParameter(4,init[4]); 
    //fitfunc->FixParameter(5,init[5]); 
    fitfunc->SetParameter(5,init[5]); 
    fitfunc->SetParameter(6,init[6]); 
    fitfunc->SetParameter(7,init[7]); 
    fitfunc->FixParameter(8,init[8]); 
  
    gr->Fit("fitfunc","r");
    fitfunc->GetParameters(&par1[0]);
    
    //TCanvas *c1 = new TCanvas("c1","c1",700,400); 
    gr->Draw("AP z");
    c1->Update();
    TPaveStats *st = (TPaveStats*)gr->FindObject("stats");
    st->SetX1NDC(0.65); //new x start position
    st->SetX2NDC(0.95); //new x end position
    st->SetY1NDC(0.3); //new x start position
    st->SetY2NDC(0.7); //new x end position
    gr->Draw("AP z");
    c1->Update();
    gr->Write();

  }
  target->Close();
  hist->Write();
  hist->Close();

  return 0;
}
