////////////////////////////////////////////////////////////////////////////////
//
// Main module for the EFFIC program
//
// This program reads a file containing two vectors, which represent "before
// cut" and "after cut" histograms. It then uses a Bayesian analysis to
// determine the posterior probability for the cut efficiency in each bin.
// A flat prior is used for the efficiency. The outputs calculated from the
// posterior probability are the most probable value of the efficiency, and
// the upper and lower edges of the shortest C% confidence interval; C is
// entered on the command line
//
// The calling syntax is:
//     EFFIC datafile conf
// where
//     datafile is the name of the data file from which we read
//     conf     is required probability content of the confidence interval
//
// Created 14 Feb 1996        Marc Paterno
////////////////////////////////////////////////////////////////////////////////
//
// Include files
#include <cmath>
#include <cstdlib>

#include "effic/effic.hpp"
#include "effic/specfunc.hpp"
#include "effic/solvers.hpp"

using namespace std;

////////////////////////////////////////////////////////////////////////////////
//
// File scope
//
namespace {
  unsigned long GLOBAL_k; // used to pass k[i] into equations
  unsigned long GLOBAL_N; // used to pass N[i] into equations
  double CONFLEVEL;        // confidence level for the interval
}

////////////////////////////////////////////////////////////////////////////////
//
// Helper functions


// beta_ab(a,b,k,N) calculates the fraction of the area under the
// curve x^k*(1-x)^(N-k) between x=a and x=b

double beta_ab(double a, double b, int k, int N) {
  if (a == b) return 0;    // don't bother integrating over zero range
  int c1 = k+1;
  int c2 = N-k+1;
  return ibetai(c1,c2,b)-ibetai(c1,c2,a);
}

// searchupper(low,k,N,c) integrates the binomial distribution with
// parameters k,N, and determines what is the upper edge of the
// integration region which starts at low which contains probability
// content c. If an upper limit is found, the value is returned. If no
// solution is found, -1 is returned.

double searchupper(double low, int k, int N, double c) {
  // check to see if there is any solution by verifying that the integral up
  // to the maximum upper limit (1) is greater than c
  double integral = beta_ab(low, 1.0, k, N);
  if (integral == c) return 1.0;		// lucky -- this is the solution
  if (integral < c) return -1.0;    // no solution exists
  double too_high = 1.0;             // upper edge estimate
  double too_low = low;
  double test;

  // use a bracket-and-bisect search
  // now we loop 20 times, to end with a root guaranteed accurate to better
  // than 0.1%
  for (int loop=0; loop<20; loop++) {
    test = 0.5*(too_low + too_high);
    integral = beta_ab(low, test, k, N);
    if (integral > c)  too_high = test;
    else too_low = test;
  }
  return test;
}

// searchlower(high,k,N,c) integrates the binomial distribution with
// parameters k,N, and determines what is the lower edge of the
// integration region which ends at high, and which contains
// probability content c. If a lower limit is found, the value is
// returned. If no solution is found, the -1 is returned.

double searchlower(double high, int k, int N, double c) {
  // check to see if there is any solution by verifying that the integral down
  // to the minimum lower limit (0) is greater than c
  double integral = beta_ab(0.0, high, k, N);
  if (integral == c) return 0.0;		// lucky -- this is the solution
  if (integral < c) return -1.0;      // no solution exists
  double too_low = 0.0; // lower edge estimate
  double too_high = high;
  double test;

  // use a bracket-and-bisect search
  // now we loop 20 times, to end with a root guaranteed accurate to better
  // than 0.1%
  for (int loop=0; loop<20; loop++) {
    test = 0.5*(too_high + too_low);
    integral = beta_ab(test, high, k, N);
    if (integral > c)  too_low = test;
    else too_high = test;
  }
  return test;
}

//
// interval(low) return the length of the interval starting at low
// that contains CONFLEVEL of the x^GLOBAL_k*(1-x)^(GLOBAL_N-GLOBAL_k)
// distribution.
// If there is no sufficient interval starting at low, we return 2.0
double interval(double low) {
  double high = searchupper(low, GLOBAL_k, GLOBAL_N, CONFLEVEL);
  if (high == -1.0) return 2.0; //  so that this won't be the shortest interval
  return (high - low);
}

////////////////////////////////////////////////////////////////////////////////
//
// Main driver routine
void effic(int k, int N, double conflevel, 
	   double& mode, double& low, double& high)
{
  // Calculate the most probable value for the posterior cross section.
  // This is easy, 'cause it is just k/N
  double efficiency = (double)k/N;
  
  // Calculate the shortest central confidence interval containing the required
  // probability content.

  // interval(low) returns the length of the interval starting at low
  // that contains CONFLEVEL probability. We use Brent's method,
  // except in two special cases: when k=0, or when k=N
  
  double low_edge;
  double high_edge;
  
  if (k == 0)
    {
      low_edge = 0.0;
      high_edge = searchupper(low_edge, k, N, conflevel);
    }
  else if (k == N)
    {
      high_edge = 1.0;
      low_edge = searchlower(high_edge, k, N, conflevel);
    }
  else
    {
      GLOBAL_k = k;
      GLOBAL_N = N;
      CONFLEVEL = conflevel;
      brent(0.0, 0.5, 1.0, interval, 1.0e-9, &low_edge);
      high_edge = low_edge + interval(low_edge);
    }

  // print output
  mode = efficiency;
  low = low_edge;
  high = high_edge;
}

