///////////////////////////////////////////////////////////////////////////////
//
// Implementation file for the special functions library.
// Adapted from Numerical Recipes in C, 2nd edition.
// Translated to C++ by by Marc Paterno
//
//

#include <cmath>
#include <stdexcept>
#include <sstream>

#include "effic/specfunc.hpp"


using namespace std;


//
// gammln() calculates the natural log of the gamma function
//
double gammln(double xx) {
  static double cof[6]={
    76.18009172947146,-86.50532032941677,
    24.01409824083091,-1.231739572450155,
    0.1208650973866179e-2,-0.5395239384953e-5};

  double y = xx;
  double x = xx;
  double tmp = x + 5.5;
  tmp -= (x+0.5)*log(tmp);
  double ser=1.000000000190015;
  for (int j=0;j<=5;j++) ser += cof[j]/++y;
  return -tmp+log(2.5066282746310005*ser/x);
}

//
// factln() calculates the natural log of the factorial function
//
double factln(unsigned n) {
  static double a[101];
  if (n <= 1) return 0.0;
  if (n <= 100) return a[n] ? a[n] : (a[n]=gammln(n+1.0));
  else return gammln(n+1.0);
}

//
// beta(z,w) calculates the complete beta function
//
double beta(double z, double w) {
  return exp(gammln(z)+gammln(w)-gammln(z+w));
}

//
// betacf() evaluates the continued fraction for the incomplete beta function
// calculation in ibeta(); uses modified Lentz's method
//
double betacf(double a, double b, double x) {
  const int MAXIT = 800;
  const double EPS = 3.0e-7;
  const double FPMIN = 1.0e-30;
  int m;

  double qab=a+b;
  double qap=a+1.0;
  double qam=a-1.0;
  double c=1.0;
  double d=1.0-qab*x/qap;
  if (abs(d) < FPMIN) d=FPMIN;
  d=1.0/d;
  double h=d;
  for (m=1;m<=MAXIT;m++) {
    int m2=2*m;
    double aa=m*(b-m)*x/((qam+m2)*(a+m2));
    d=1.0+aa*d;
    if (abs(d) < FPMIN) d=FPMIN;
    c=1.0+aa/c;
    if (abs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    h *= d*c;
    aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
    d=1.0+aa*d;
    if (abs(d) < FPMIN) d=FPMIN;
    c=1.0+aa/c;
    if (abs(c) < FPMIN) c=FPMIN;
    d=1.0/d;
    double del=d*c;
    h *= del;
    if (abs(del-1.0) < EPS) break;
  }
  if (m > MAXIT) {
    ostringstream oss;
    oss << "a or b too big, or MAXIT too small, in betacf()\n"
	<< "a = " << a << " b = " << b << " MAXIT = " << MAXIT << endl;
    throw runtime_error(oss.str());
  }
  return h;
}

//
// ibetai(a,b,x) calculates the incomplete beta function  I_x(a,b); this is
// the incomplete beta function divided by the complete beta function
//
double ibetai(double a, double b, double x) {
  double bt;
  if (x < 0.0 || x > 1.0) {
    ostringstream oss;
    oss << "Illegal x in routine ibetai: x = " << x << endl;
    throw runtime_error(oss.str());
  }
  if (x == 0.0 || x == 1.0)
    bt=0.0;
  else
    bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));

  if (x < (a+1.0)/(a+b+2.0))
    return bt*betacf(a,b,x)/a;
  else
    return 1.0-bt*betacf(b,a,1.0-x)/b;
}

//
// betai(a,b,x) calculates the incomplete beta function B_x(a,b), as defined
// in Gradshteyn and Ryzhik (4th edition) 8.391
//
double betai(double a, double b, double x) {
  return ibetai(a,b,x)*beta(a,b);
}
